/*
 * @copyright		© Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/customform/inc/customformpreview
 */

define("de_epages/customform/inc/customformpreview", [
	"jquery",
	"de_epages",
	"$dict!../dictionary",
	"$tmpl!./customformpreview",

	"jquery/tmpl",
	"jquery/ui/sortable",
	"ep/ui/dialog",
	"ep/alert",
	"ep/validate",
	"ep/ui/validate",
	"de_epages/customform/ui/customform"
], function ($, de_epages, customformDict, tmplCustomform) {

	return {
		customformPreviewHandler: function(options){
			/*
			 * @dictionary		ep.dict
			 *
			 * @translation		{Type}
			 *								{MandatoryStar}
			 *								{yes}
			 *								{no}
			 *								{Apply}
			 *								{Cancel}
			 *								{Delete}
			 *								{Edit}
			 *								{Move}
			 *
			 *
			 * @dictionary		de_epages.customform.dictionary
			 *
			 * @translation		{FieldName}
			 *								{MandatoryForUsers}
			 *								{AdditionalFormFieldText}
			 *								{AddFormField}
			 *								{ConfirmDelete}
			 */

			var o = $.extend({
//					ViewAction: undefined,
//					ObjectID: undefined,
//					element: undefined,					dialogContentId: 'epCustomFormWrapperId',
					dialogEditId	: 'editDialogContent'
				}, options),

				// translations
				translationsDialog = {
					tMandatoryStar: customformDict.translate('MandatoryStar'),
					tDelete: customformDict.translate('Delete'),
					tEdit: customformDict.translate('Edit'),
					tMove: customformDict.translate('Move'),
					tConfirmDelete: customformDict.translate('ConfirmDelete')
				},

				radioMarkup = '<span><input type="radio" name="${Alias}" value="${OptionName}" class="ep-js" data-js="ep.uiInput()" /><label>${OptionName}</label><span>',
				selectMarkup = '<option name="${Alias}" value="${OptionName}">${OptionName}</option>',

				listElements,

				// initialize
				init = function(){
					// if an element is defined
					if(o.element !== undefined){

						listElements = o.element.children('dl');

						listElements.each(function(index, elem){
							var $elem = $(elem),
								termElement = $elem.find('dt'),
								descriptionElement = $elem.find('dd'),
								formFieldID = $elem.data('objectid'),
								formFieldAlias = $elem.data('formfieldalias');

							// set position relative so that the toolbox can set absolute
							descriptionElement
								.css({
									'position': 'relative'
								});

							// include template for toolbox (delete, move, edit)
							tmplCustomform(translationsDialog).appendTo(descriptionElement);

							var toolboxElement = descriptionElement.find('.customFormTools'),
								editNode = toolboxElement.find('.EditNode'),
								deleteNode = toolboxElement.find('.DeleteNode');

							// if current field is for the user email -> remove delete and edit buttons
							if($elem.data('formfieldalias')== "EMailFormField"){
								editNode.remove();
								deleteNode.remove();
							}else{
								deleteNode.on('click', function(){
									ep.alert({
										title: translationsDialog.tDelete,
										okTitle: customformDict.translate('{Delete}'),
										type: 'confirmation',
										size: 'l',
										content: translationsDialog.tConfirmDelete,
										close: function(event){
											// do something on close
										},
										ok: function(event){
											ep.ajax({
												url: '?',
												type: 'post',
												data: {
													'ChangeAction': 'JSONDeleteField',
													'ViewAction': 'JSONViewResponse',
													'ObjectID': formFieldID
												}
											})
											.done(function(data, status, jqXHR){
												// ToDo: some stuff if request was successfull
												//console.debug("arguments", arguments);

//												$elem.fadeOut(500, function(){
//													$elem.remove();
//												});

												$elem.animate({
											    opacity: 0,
											    height: 0
											  }, 500, function(){
											    // animation complete
													$elem.remove();
											  });
											})
											.fail(function(jqXHR, status, error){
												// uncomment next line for debug use
												//console.log("fail:", error);
											});
										},
										cancel: function(event){
											// do something on abort
										}
									});
								});

								// nodes which should be able to open edit dialog when clicked
								var editCollection = $.merge(editNode, termElement);

								// add some data so that the customformUiCustomform-widget works
								//editNode.data({
								editCollection.data({
									'objectid': $elem.data('objectid'),
									'formfieldid': $elem.data('formfieldid')
								});

								// add widget to button to provide edit functionality (dialog, edit fields ...)
								de_epages(editCollection).customformUiCustomform({
								//de_epages(editNode).customformUiCustomform({
      						isPreview: true,
									languageId: o.languageId,
									languages: o.languages,

									// callback -> called after dialog data was saved successully (self = customformUiCustomform-widget)
									afterSaveDialog: function(self, data){
										var label, mandatory;

										// set or remove mandatory star
										if(data.Alias != "CheckBoxFormField"){
											label = termElement.find('label');
											//if(parseInt(self.dialogContentHolder.find('input[name=IsMandatory]:checked').val()) == 1){
											if(data.IsMandatory){
												termElement.append('<span class="MandatoryStar">*</span>');
											}else{
												termElement.children('.MandatoryStar').remove();
											}
										}else{
											label = descriptionElement.find('label:first');
											mandatory = descriptionElement.find('span.MandatoryStar');

											if(mandatory.length){
												//mandatory[data.IsMandatory ? 'show' : 'hide']();
												mandatory.css('display', data.IsMandatory ? '' : 'none');
											}else if(data.IsMandatory){
												label.after('<span class="MandatoryStar">'+translationsDialog.tMandatoryStar+'</span>');
											}
										}

										// set field name if necessary
										var rewriteFieldName = data.FieldName;
										if(self._fieldName != rewriteFieldName){
											if(data.Alias != "CheckBoxFormField"){
												if (label.length)  {
													label.html(rewriteFieldName);
												} else {
													descriptionElement.find('.customFormCaption').html(rewriteFieldName!=null ? rewriteFieldName : "&nbsp;");
												}
											}else{
												label.html(rewriteFieldName);
											}
										}

										// if dialog contains a field for additional text
										if($('#rewriteAdditionalTextId').length){
											// set additional text if necessary
											var rewriteAdditionalText = $('#rewriteAdditionalTextId').val();
											if(rewriteAdditionalText != ""){
												var additionalInfo = descriptionElement.find('.AdditionalInfo');
												// if already an element for additional text exists, use it otherwise add new element
												if (additionalInfo.length)  {
													additionalInfo.html(rewriteAdditionalText);
												} else {
													descriptionElement.append('<div class="AdditionalInfo">'+rewriteAdditionalText+'</div>');
												}
											}else{
												descriptionElement.children('.AdditionalInfo').remove();
											}
										}

										// SelectBoxFormField
										if(data.Alias == "SelectBoxFormField" && data.HasOptions){
											// compile the markup for select options as a named template
											$.template("selectTemplate", selectMarkup);

											var selectField = descriptionElement.find('select'),
												firstOption = selectField.children(':first').clone();

											// clear select group and set first option
											selectField[0].options.length = 0;
											selectField.append(firstOption);

											// iterate over all options
											$.each(data.Options, function(index, option){
												$.tmpl('selectTemplate', option).appendTo(selectField);
											});
										}

										// RadioButtonFormField
										if(data.Alias == "RadioButtonFormField" && data.HasOptions){
											var radioHolder = descriptionElement.find('div.customFormRadioHolder'),
												cloneRadioElement = radioHolder.children('span:first').clone();

											radioHolder.empty();

											// compile the markup for radio buttons as a named template
											$.template("radioTemplate", radioMarkup);

											// iterate over all options
											$.each(data.Options, function(index, option){
												$.tmpl('radioTemplate', option).appendTo(radioHolder);
											});

											ep(radioHolder).find('input').metaparse();

										}
									}
								});
							}

							$elem
								.on('mouseenter', function(){
									$(this).addClass('activeElement');

									toolboxElement.css({
										display: 'block'
									});
								})
								.on('mouseleave', function(){
									$(this).removeClass('activeElement');

									toolboxElement.css({
										display: 'none'
									});
								});
						});

						// make form list sortable
						o.element
							.sortable({
								containment: o.element.closest('form'),
								axis: 'y',
								cursor: 'crosshair',
								placeholder: 'ui-state-highlight',
								forcePlaceholderSize: true,
								handle: 'span.DndNode',
								stop: function(event, ui){
									var prevItem = ui.item.prev(),
										// calculate new position of dragged element
										currentPosition = prevItem.length? (prevItem.data('position')+10) : 10;

									// if element was dragged to new position
									if(currentPosition != ui.item.data('position')){
										ui.item.data('position', currentPosition);
										ui.item.find('input[name=Position]').val(currentPosition);

										ui.item.nextAll().each(function(key, listItem){
											var $this = $(this);

											// calculate position for next element
											currentPosition += 10;
											$this.data('position', currentPosition);
											$this.find('input[name=Position]').val(currentPosition);
										});


										// built request string
										var requestString = "ViewAction=JSONViewResponse&ChangeAction=JSONSaveFields&ObjectID="+o.ObjectID,
										//var requestString = "ChangeAction=JSONSaveFields&ObjectID="+o.ObjectID,
											requestArray = [
												{name: "ChangeAction", value: "JSONSaveFields"},
												{name: "ViewAction", value: "JSONViewResponse"},
												{name: "ObjectID", value: o.ObjectID}
											];

										listElements.each(function(index, elem){
											//requestString += "&Position="+$(elem).data('position')+"&PositionObjectID="+$(elem).data('positionobjectid');
											requestArray.push(
												{name: "Position", value: $(elem).data('position')},
												{name: "PositionObjectID", value: $(elem).data('positionobjectid')}
											);
										});

										// save new positions
										ep.ajax({
											url: '?',
											//dataType: 'json',
											type: 'post',
											//data: requestString
											data: requestArray
										})
										.done(function(){
											// ToDo: some stuff if request was successfull
											//console.debug("arguments", arguments);
										})
										.fail(function(){
											// ToDo: some stuff if request was not successfull
											//console.debug("arguments (fail)", arguments);
										});
									}
								}
							})
							.disableSelection();

					}
				};

			init();
		}
	};

});
