/*
 * @copyright		© Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/shipping/inc/newdeliverydialog
 */

define("de_epages/shipping/inc/newdeliverydialog", [
	"jquery",
	"ep",

	"jquery/ui/draggable",
	"ep/ajax",
	"ep/alert",
	"ep/ui/dialog"
], function ($, ep) {

	var newdeliverydialog;

	return (newdeliverydialog = {
		ShopObjectId: null,

		NewDeliveryMethodDialog: null,

		CurrentShippingFulfillmentItem: null,

		CurrentShippingTypeList: null,

		CurrentShippingTypeItem: null,

		open: function (objectId) {
			this.ShopObjectId = objectId;
			$('.Dialog')
				.busy('show');

			ep.ajax({
				url: "?ViewAction=MBO-AddNewDeliveryMethodDialog&ObjectID=" + this.ShopObjectId
			})
			.done(function (data) {
				newdeliverydialog.show(data);
			})
			.fail(function () {
				newdeliverydialog.ajaxError();
			})
			.always(function () {
				$('.Dialog')
					.busy('hide');
			});
		},

		show: function (data) {
			if (this.NewDeliveryMethodDialog == null) {
				this.NewDeliveryMethodDialog = $('#AddShippingDialog')
					.html(data)
					.uiDialog({
					title: $('#AddShippingDialogLabels > #DialogTitle')
						.html(),
					modal: true,
					autoOpen: false,
					draggable: true,
					resizable: false,
					width: 700,
					maxHeight: 600,
					buttons: [{
            			cssClass: 'ep-button ep-button-save',
						text: $('#AddShippingDialogLabels > #OkButton')
							.html(),
						click: function () {
							newdeliverydialog.submit();
						}
					}, {
            			cssClass: 'ep-button ep-button-cancel',
						text: $('#AddShippingDialogLabels > #CloseButton')
							.html(),
						click: function () {
							newdeliverydialog.close();
						}
					}],
					open: function () {
						$('#AddShippingDialog')
							.keypress(function (e) {
								if (e.keyCode == $.ui.keyCode.ENTER) {
									e.preventDefault();
									$(this)
										.next()
										.find('button:eq(0)')
										.trigger('click');
								}
							});
					}
				});
			}
			else {
				this.NewDeliveryMethodDialog.html(data);
			}
			this.init();
			this.NewDeliveryMethodDialog.uiDialog('open');
		},

		submit: function () {
			this.lock(true);
			this.submitShippingFulfillment();
			this.submitShippingType();
			var self = this;

			ep.ajax({
				url: "?ViewAction=MBO-AddNewDeliveryMethodDialog&ObjectID=" + this.ShopObjectId,
				type: "POST",
				data: $('#NewDeliveryMethodForm')
					.serialize()
			})
			.done(function (data) {
				newdeliverydialog.reload(data);
			})
			.fail(function () {
				newdeliverydialog.ajaxError();
			})
			.always(function () {
				newdeliverydialog.lock(false);
			});
		},

		reload: function (data) {
			var newShippingMethodId = null;
			var gotoView = null;
			var regexp = new RegExp('\\bid="NewShippingMethodID" value="(\\d+)"');
			var result = regexp.exec(data);
			if (result != null) {
				newShippingMethodId = result[1];
				regexp = new RegExp('\\bid="GotoView" value="(.+)"');
				result = regexp.exec(data);
				gotoView = result[1];
			}
			if (newShippingMethodId != null) {
				this.NewDeliveryMethodDialog.uiDialog('close');
				$('.Dialog')
					.busy('show');
				document.location = '?ViewAction=' + gotoView + '&ObjectID=' + newShippingMethodId;
			}
			else {
				this.NewDeliveryMethodDialog.html(data);
				this.init();
				this.NewDeliveryMethodDialog.uiDialog('open');
			}
		},

		init: function () {
			this.onLoadShippingType();
			this.onLoadShippingFulfillment();
		},

		close: function () {
			this.NewDeliveryMethodDialog.uiDialog('close');
		},

		lock: function (lock) {
			if (this.NewDeliveryMethodDialog != null) {
				this.NewDeliveryMethodDialog.parent()
					.busy((lock) ? 'show' : 'hide');
			}
		},

		onChangeShippingFulfillmentList: function (object,ShippingProviderTypeID) {
			if (this.CurrentShippingFulfillmentItem != null) {
				this.CurrentShippingFulfillmentItem.hide();
			}
			this.CurrentShippingFulfillmentItem = $('#ShippingFulfillmentInfo_' + ShippingProviderTypeID);
			this.CurrentShippingFulfillmentItem.removeClass('HideElement')
				.show();
			this.showShippingTypeList(object.value);
		},

		submitShippingFulfillment: function () {},

		onLoadShippingFulfillment: function () {
			var element = $('#PreviousShippingFulfillmentTypeID');
			var previousId = element.val();
			if (previousId == '') {
				element.val(0);
			}
			$('#NewShippingFulfillmentTypeID option[value=' + element.val() + ']')
				.prop('selected', true);
			this.onChangeShippingFulfillmentList(element[0]);
		},

		onChangeShippingTypeList: function (object) {
			if (this.CurrentShippingTypeItem != null) {
				this.CurrentShippingTypeItem.hide();
			}
			if (object.value.length) {
				this.CurrentShippingTypeItem = $('#ShippingTypeInfo_' + object.value);
				this.CurrentShippingTypeItem.removeClass('HideElement')
					.show();
			}
			else {
				this.CurrentShippingTypeItem = null;
			}
		},

		showShippingTypeList: function (id_logistic) {
			var pattern = /[0-9]*/;
			var listId = '#ShippingType_' + pattern.exec(id_logistic);
			var listObject = $(listId);
			if (listObject.length != 0) {
				var currentListId = '#' + this.CurrentShippingTypeList[0].id;
				if (currentListId != listId) {
					this.CurrentShippingTypeList.hide();
					this.CurrentShippingTypeList = listObject;
					this.CurrentShippingTypeList.removeClass('HideElement')
						.show();
					this.onChangeShippingTypeList(this.CurrentShippingTypeList[0]);
				}
			}
		},

		submitShippingType: function () {
			$('#NewShippingTypeID')
				.val(this.CurrentShippingTypeList[0].value);
		},

		onLoadShippingType: function () {
			this.CurrentShippingTypeList = $('#ShippingType_0');
			var previousId = $('#PreviousShippingTypeID')
					.val();
			if (previousId != '') {
				$('.ShippingTypeList option[value=' + previousId + ']')
					.prop('selected', true);
			}
		},

		ajaxError: function () {
			ep.alert({
				title: $('#AddShippingDialogLabels > #AlertErrorTitle')
					.html(),
				content: $('#AddShippingDialogLabels > #AlertErrorMessage')
					.html(),
				type: 'warn',
				ok: function (event) {}
			});
		}
	});

});
